import numpy as np
import os
import epicure.epicuring as epi

def test_get_free_label():
    ## test from a skeletonized movie
    test_mov = os.path.join(".", "data_test", "003_crop.tif")
    test_seg = os.path.join(".", "data_test", "003_crop_epyseg.tif")
    epic = epi.EpiCure()
    epic.load_movie(test_mov)
    #epic.viewer = napari.Viewer(show=False)
    epic.go_epicure("epics", test_seg)
    assert len(epic.get_free_labels(5)) == 5
    assert len(epic.get_free_labels(15)) == 15
    epic = None
    
    ## test from a label image
    test_img = os.path.join(".", "data_test", "170119_crop.tif")
    test_seg = os.path.join(".", "data_test", "170119_crop_mask.tif")
    epic = epi.EpiCure()
    epic.load_movie(test_img)
    epic.go_epicure("epics", test_seg)
    assert (5 == epic.get_free_labels(5)[0])
    assert 5 == epic.get_free_labels(1)[0]
    assert len(epic.get_free_labels(5)) == 5
    assert len(epic.get_free_labels(15)) == 15
    epic = None




import numpy as np
import os
import epicure.epicuring as epi
import epicure.Utils as ut
from unittest.mock import Mock
import napari
from vispy import keys


def test_suspect_frame():
    test_img = os.path.join(".", "data_test", "area3_t100-101.tif")
    test_seg = os.path.join(".", "data_test", "area3_epyseg-t100-101.tif")
    viewer = napari.Viewer(show=False)
    epic = epi.EpiCure(viewer)
    resaxis, resval = epic.load_movie(test_img)
    epic.set_chanel(1, 1)
    assert epic.viewer is not None
    epic.go_epicure("test_epics", test_seg)
    
    segedit = epic.inspecting
    assert segedit is not None
    segedit.min_area.setText("50")
    segedit.event_area_threshold()
    assert "Events" in epic.viewer.layers
    outlier = epic.viewer.layers["Events"]
    assert len(outlier.data)>=10
    assert outlier.data[1][0] == 0
    
    nsus = len(outlier.data)
    segedit.fintensity_out.setText("0.5")
    segedit.event_intensity(True)
    assert len(outlier.data) > (nsus+5)

def test_suspect_track():
    test_img = os.path.join(".", "data_test", "003_crop.tif")
    test_seg = os.path.join(".", "data_test", "003_crop_epyseg.tif")

    ## load and initialize
    epic = epi.EpiCure()
    epic.load_movie(test_img)
    epic.go_epicure("test_epics", test_seg)
    
    track = epic.tracking
    
    # default tracking
    susp = epic.inspecting
    assert susp.nb_events() == 0
    track.do_tracking()
    ## test basics
    assert susp.nb_events() == susp.nb_type("division")
    nev = susp.nb_events()
    susp.add_event( (5,50,50), 10, "test" )
    assert susp.nb_events() == (nev+1)
    ## test default parameter inspection
    susp.check_size.setChecked( False )
    susp.check_length.setChecked( False )
    susp.inspect_tracks()
    assert susp.nb_events() > 50
    assert susp.nb_events() < 100
    ## test minimum track length inspection
    susp.check_length.setChecked( True )
    susp.min_length.setText("5")
    nmin_prev =  susp.nb_events()
    susp.inspect_tracks()
    nmin =  susp.nb_events()
    assert nmin > nmin_prev
    assert nmin > 100 
    ## test reset all
    susp.reset_all_events()
    assert susp.nb_events() == 0
    ## Test reloading the divisions from the track graph
    susp.get_divisions()
    assert susp.nb_events() == nev
    ## Track feature change test
    susp.check_size.setChecked( True )
    susp.inspect_tracks()
    assert susp.nb_events() > nmin

def test_boundaries():
    test_img = os.path.join(".", "data_test", "003_crop.tif")
    test_seg = os.path.join(".", "data_test", "003_crop_epyseg.tif")

    ## load and initialize
    epic = epi.EpiCure()
    epic.load_movie(test_img)
    epic.go_epicure("test_epics", test_seg)

    ## check that doesn't find any boundary cells (touching background)
    epic.inspecting.get_boundaries_cells()
    assert len(epic.inspecting.boundary_cells[0]) == 0

    ## remove the border cells, so now should find boundary cells
    epic.editing.remove_border()
    epic.inspecting.get_boundaries_cells()
    assert len(epic.inspecting.boundary_cells[0]) > 20

#test_suspect_track()
#test_boundaries()



"""
 Track with btrack package
 https://github.com/quantumjot/btrack/tree/main
"""

import btrack as bt
        self.track_choice.addItem("BTrack")
        self.create_btrack()
        layout.addWidget(self.gBTrack)

############ BTrack option

    def create_btrack(self):
        """ GUI of the BTrack option """
        self.gBTrack, gb_layout = wid.group_layout( "BTrack" )
        featlab = wid.label( "Include features:" )
        gb_layout.addWidget(featlab)
        self.check_area = wid.check( "Area", True, "Include area of the label in the tracking" )
        gb_layout.addLayout(self.check_area)
        self.gBTrack.setLayout(gb_layout)

    def go_btrack( self, start_frame, end_frame ):
        """ Run BTrack on the selected frames """
        self.btrack = BTrack( self.track, self.epicure )
        props = ()
        if self.check_area.isChecked():
            props = props + ("area",)
        btobj = self.btrack.prepare_objects(self.epicure.seg[start_frame:end_frame+1], props )
        self.btrack.track( btobj )



class BTrack():

    def __init__(self, track, epic):
        self.track = track
        self.epicure = epic
        self.tracker = bt.BayesianTracker()
        bt.configure_from_file("btrack_config.json")
        bt.max_search_radius = 10

    def prepare_objects( self, movie, region_props ):
        """ Prepare the movie (stack of label images) for BTrack format """
        obj = bt.utils.segmentation_to_objects( movie, properties=region_props )
        print(obj)
        return obj



    def track(self, obj, **kwargs):
        """ Track the regions """
        return self.tracker.track(regions, **kwargs)


import numpy as np
import os
import epicure.epicuring as epi

def test_load_movie():
    test_img = os.path.join(".", "data_test", "003_crop.tif")
    epic = epi.EpiCure()
    epic.load_movie(test_img)
    assert epic.img.shape == (11,208,426)

def test_load_movie_with_chanel():
    test_img = os.path.join(".", "data_test", "area3_t100-101.tif")
    test_seg = os.path.join(".", "data_test", "area3_epyseg-t100-101.tif")
    epic = epi.EpiCure()
    resaxis, resval = epic.load_movie(test_img)
    assert epic.img.shape == (2,2,384,394)
    assert resaxis == 1
    assert resval == 2
    assert np.mean(epic.img)>=268
    epic.set_chanel(1, 1)
    assert np.mean(epic.img)<268
    assert np.mean(epic.img)>100

def test_load_segmentation():
    test_seg = os.path.join(".", "data_test", "003_crop_epyseg.tif")
    epic = epi.EpiCure()
    epic.load_segmentation(test_seg)
    assert epic.seg.shape == (11,208,426)
    assert np.max(epic.seg) == 2842

def test_suggest():
    test_img = os.path.join(".", "data_test", "003_crop.tif")
    epic = epi.EpiCure()
    epic.load_movie(test_img)
    segfile = epic.suggest_segfile("epics")
    assert segfile is None
    test_img = os.path.join(".", "data_test", "170119_crop.tif")
    epic.load_movie(test_img)
    segfile = epic.suggest_segfile("epics")
    assert segfile == os.path.join(".", "data_test", "epics", "170119_crop_labels.tif")

def test_init_epic():
    epic = epi.EpiCure()
    assert epic.img is None

#if __name__ == "__main__":
#    test_init_epic()
#    print("********* Test cure completed ***********")

#test_load_movie_with_chanel()
